#ifndef UPNPSUBSCRIPTIONREQUEST_H
#define UPNPSUBSCRIPTIONREQUEST_H

/*!
 * \file
 *
 * \brief Header file for UpnpSubscriptionRequest methods.
 *
 * Do not edit this file, it is automatically generated. Please look at
 * generator.c.
 *
 * \author Marcelo Roberto Jimenez
 */
#include <stdlib.h> /* for size_t */

#include "UpnpGlobal.h" /* for UPNP_EXPORT_SPEC */

#include "UpnpString.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*!
 * UpnpSubscriptionRequest
 */
typedef struct s_UpnpSubscriptionRequest UpnpSubscriptionRequest;

/*! Constructor */
UPNP_EXPORT_SPEC UpnpSubscriptionRequest *UpnpSubscriptionRequest_new(void);
/*! Destructor */
UPNP_EXPORT_SPEC void UpnpSubscriptionRequest_delete(
	UpnpSubscriptionRequest *p);
/*! Copy Constructor */
UPNP_EXPORT_SPEC UpnpSubscriptionRequest *UpnpSubscriptionRequest_dup(
	const UpnpSubscriptionRequest *p);
/*! Assignment operator */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_assign(
	UpnpSubscriptionRequest *p, const UpnpSubscriptionRequest *q);

/*! UpnpSubscriptionRequest_get_ServiceId */
UPNP_EXPORT_SPEC const UpnpString *UpnpSubscriptionRequest_get_ServiceId(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_set_ServiceId */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_set_ServiceId(
	UpnpSubscriptionRequest *p, const UpnpString *s);
/*! UpnpSubscriptionRequest_get_ServiceId_Length */
UPNP_EXPORT_SPEC size_t UpnpSubscriptionRequest_get_ServiceId_Length(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_get_ServiceId_cstr */
UPNP_EXPORT_SPEC const char *UpnpSubscriptionRequest_get_ServiceId_cstr(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_strcpy_ServiceId */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_strcpy_ServiceId(
	UpnpSubscriptionRequest *p, const char *s);
/*! UpnpSubscriptionRequest_strncpy_ServiceId */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_strncpy_ServiceId(
	UpnpSubscriptionRequest *p, const char *s, size_t n);
/*! UpnpSubscriptionRequest_clear_ServiceId */
UPNP_EXPORT_SPEC void UpnpSubscriptionRequest_clear_ServiceId(
	UpnpSubscriptionRequest *p);

/*! UpnpSubscriptionRequest_get_UDN */
UPNP_EXPORT_SPEC const UpnpString *UpnpSubscriptionRequest_get_UDN(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_set_UDN */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_set_UDN(
	UpnpSubscriptionRequest *p, const UpnpString *s);
/*! UpnpSubscriptionRequest_get_UDN_Length */
UPNP_EXPORT_SPEC size_t UpnpSubscriptionRequest_get_UDN_Length(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_get_UDN_cstr */
UPNP_EXPORT_SPEC const char *UpnpSubscriptionRequest_get_UDN_cstr(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_strcpy_UDN */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_strcpy_UDN(
	UpnpSubscriptionRequest *p, const char *s);
/*! UpnpSubscriptionRequest_strncpy_UDN */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_strncpy_UDN(
	UpnpSubscriptionRequest *p, const char *s, size_t n);
/*! UpnpSubscriptionRequest_clear_UDN */
UPNP_EXPORT_SPEC void UpnpSubscriptionRequest_clear_UDN(
	UpnpSubscriptionRequest *p);

/*! UpnpSubscriptionRequest_get_SID */
UPNP_EXPORT_SPEC const UpnpString *UpnpSubscriptionRequest_get_SID(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_set_SID */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_set_SID(
	UpnpSubscriptionRequest *p, const UpnpString *s);
/*! UpnpSubscriptionRequest_get_SID_Length */
UPNP_EXPORT_SPEC size_t UpnpSubscriptionRequest_get_SID_Length(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_get_SID_cstr */
UPNP_EXPORT_SPEC const char *UpnpSubscriptionRequest_get_SID_cstr(
	const UpnpSubscriptionRequest *p);
/*! UpnpSubscriptionRequest_strcpy_SID */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_strcpy_SID(
	UpnpSubscriptionRequest *p, const char *s);
/*! UpnpSubscriptionRequest_strncpy_SID */
UPNP_EXPORT_SPEC int UpnpSubscriptionRequest_strncpy_SID(
	UpnpSubscriptionRequest *p, const char *s, size_t n);
/*! UpnpSubscriptionRequest_clear_SID */
UPNP_EXPORT_SPEC void UpnpSubscriptionRequest_clear_SID(
	UpnpSubscriptionRequest *p);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* UPNPSUBSCRIPTIONREQUEST_H */
